//*****************************************************************************
// Copyright (c) 2006, Freescale Semiconductor
// For use on Freescale products only.
//
// File name :   	freescale_UDP_client.c
// Author:			Eric Gregori   (847) 651 - 1971
//		   			eric.gregori@freescale.com
//
// Description : 	UDP client
//*****************************************************************************
#include "ipport.h"
#include "tcpapp.h"
#include "msring.h"
#include "menu.h"
#include OSPORT_H
#include "ipport.h"
#include "libport.h"
#include "udp.h"


#define 		PORT_NUMBER			5678
#define			TEST_BUFFER			250		// *4 = 1000 bytes
#define			INTER_PACKET_DELAY	1
#define			SERVER_IP			0xC0A80101 // 192.168.1.1

//*****************************************************************************
// Declare Task Object
//*****************************************************************************
TK_OBJECT(to_freescale);
TK_ENTRY(tk_freescale);
struct inet_taskinfo freescale_task = {
      									&to_freescale,
      									"FreeScale Task",
      									tk_freescale,
      									NET_PRIORITY,
      									0x800
									};

unsigned long 	data_to_send[TEST_BUFFER];

//*****************************************************************************
// Declare a Socket structure and communications queue "msring"
//*****************************************************************************



//*****************************************************************************
//
// emg_udpsend based on tftp_udpsend
// Written by Eric Gregori
//
// Send outbuf to dest_port at ip address dest_ip
//
//*****************************************************************************
int emg_udpsend (	ip_addr dest_ip, 
					unsigned short dest_port, 
					char *outbuf, 
					int outlen
				)
{
	PACKET 		pkt2;   // packet to send & free 
	int			e;
	
	pkt2 = udp_alloc(outlen, 0);
	if(!pkt2)
		return -1;
	
	for( e=0; e<outlen; e++ )
		pkt2->nb_prot[e] = outbuf[e];
				
   	pkt2->nb_plen 	= outlen;
  	pkt2->fhost 	= dest_ip;
   	pkt2->net 		= NULL;
						  // local port
   	e = udp_send( dest_port, 0x1234,     pkt2 );

   if(e < 0)
   {
#ifdef NPDEBUG
      dprintf("tftp_udpsend(): udp_send() error %d\n", e);
#endif
      return e;
   }
   else
      return 0;
}


//*****************************************************************************
//
// Sample function for a UDP client
//
//*****************************************************************************
void emg_send_data_via_udp( void )
{
	ip_addr 		dest_ip = SERVER_IP; 
	int		len;

#if 0
	len = read_AD(0)/4;
	if( len > (TEST_BUFFER*4) )
		len = (TEST_BUFFER*4);
#endif
	len = 1450;
	
	data_to_send[0] = data_to_send[0] + 1;
#if 0	
	data_to_send[0] = (unsigned long)'ERIC';
	data_to_send[1] = read_AD(0);
	data_to_send[2] = read_AD(1);
	data_to_send[3] = read_AD(2);
	data_to_send[4] = read_AD(3);
	data_to_send[5] = read_AD(4);
	data_to_send[6] = read_AD(5);
	data_to_send[7] = read_AD(6);
	data_to_send[8] = (MCF_RTC_HOURMIN & 0x00001F00)>>8;;
	data_to_send[9] = MCF_RTC_HOURMIN & 0x0000003F;
	data_to_send[10]= MCF_RTC_SECONDS & 0x0000003F;
	
	printf( "\nsending data" );
#endif	
	
	// Send data_to_send to ip address det_ip, port PORT_NUMBER
	if( emg_udpsend( dest_ip, PORT_NUMBER, (void *)data_to_send, len ) )
		printf( "\nError sending via UDP " );
//	printf( "\nsent %d", len );	
}


//*****************************************************************************
//
// UDP client init
//
//*****************************************************************************
void emg_init_udp_client( void )
{
	unsigned int	i;
	
	for( i=0; i<TEST_BUFFER; i++ )
		data_to_send[i] = i;	
}


//*****************************************************************************
//
// UDP client init
//
//*****************************************************************************
void emg_udp_client_cleanup( void )
{
	
}


//*****************************************************************************
// The application thread works on a "controlled polling" basis: 
// it wakes up periodically and polls for work.
//
// The task could aternativly be set up to use blocking sockets,
// in which case the loops below would only call the "xxx_check()"
// routines - suspending would be handled by the TCP code.
//
//
// FUNCTION: tk_emg_http_srv
// 
// PARAM1: n/a
//
// RETURNS: n/a
//
//*****************************************************************************
TK_ENTRY(tk_freescale)
{
	while (!iniche_net_ready)
    	TK_SLEEP(1);

	emg_init_udp_client();
      
	for (;;)
	{
    	emg_send_data_via_udp();         	
      	tk_yield();          			

      	if (net_system_exit)
        	break;
   	}
   	TK_RETURN_OK();
}


//*****************************************************************************
// create_freescale_task() - Written by Eric Gregori
//		   			  		 eric.gregori@freescale.com
//
// Insert the FreeScale task into the RTOS.
//*****************************************************************************
void create_freescale_task( void )
{
	int e = 0;

   	e = TK_NEWTASK(&freescale_task);
   	if (e != 0)
   	{
      dprintf("freescale task create error\n");
      panic("create_apptasks");
   	}   	
}

